//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
#ifndef NT7108_H
#define NT7108_H


#include <avr/io.h>


//------------------------------------------------------------------------------
// Data and control ports
//------------------------------------------------------------------------------
#define NT7108_DATA_DDR         DDRB
#define NT7108_DATA_PORT        PORTB
#define NT7108_DATA_PIN         PINB

#define NT7108_CTRL_DDR         DDRC
#define NT7108_CTRL_PORT        PORTC


//------------------------------------------------------------------------------
// Control port bits
//------------------------------------------------------------------------------
#define NT7108_CS1              PC5	//ROSU
#define NT7108_CS2              PC4	//MARO
#define NT7108_RST              PC3	//NEGRU
#define NT7108_RW               PC2	//ALB
#define NT7108_DI               PC1	//GRI
#define NT7108_E                PC0	//MOV


//------------------------------------------------------------------------------
// Instructions
//------------------------------------------------------------------------------
#define NT7108_DISPLAY_ON       0x3f
#define NT7108_DISPLAY_OFF      0x3e

#define NT7108_Y_ADDRESS(y)     (0x40 | (((unsigned char)(y)) & 0x3f))
#define NT7108_X_ADDRESS(x)     (0xb8 | (((unsigned char)(x)) & 0x07))
#define NT7108_Z_ADDRESS(z)     (0xc0 | (((unsigned char)(z)) & 0x3f))

#define NT7108_ADDRESS(y)       NT7108_Y_ADDRESS(y)
#define NT7108_PAGE(x)          NT7108_X_ADDRESS(x)
#define NT7108_START_LINE(z)    NT7108_Z_ADDRESS(z)


//------------------------------------------------------------------------------
// Status read
//------------------------------------------------------------------------------
#define NT7108_IS_BUSY(s)       ((s) & (1 << PB7))
#define NT7108_IS_ON(s)         ((s) & (1 << PB5))
#define NT7108_IS_OFF(s)        (!IS_ON(s))
#define NT7108_IS_RESET(s)      ((s) & (1 << PB4))


//------------------------------------------------------------------------------
#define NT7108_DELAY() __asm__ __volatile__ ("nop\nnop\nnop\nnop\nnop\nnop\nnop\nnop")


//------------------------------------------------------------------------------
//
//------------------------------------------------------------------------------
void          NT7108_Initialize      (void);
void          NT7108_WriteInstruction(unsigned char ChipSelect, unsigned char Data);
void          NT7108_WriteData       (unsigned char ChipSelect, unsigned char Data);
unsigned char NT7108_ReadStatus      (unsigned char ChipSelect);
unsigned char NT7108_ReadData        (unsigned char ChipSelect);


#endif

