/*
 *  Postoaca Andrei Vlad 332CC
 *	Proiect PM - Minesweeper
 */

#define F_CPU         16000000
#define SCREEN_HEIGHT 64
#define SCREEN_WIDTH  128

#define SQUARE_SIDE 8
#define SCREEN_WIDTH_SQUARENO 14
#define SCREEN_HEIGHT_SQUARENO 7

#define MINE_COUNT 20

#define BUFLEN 16

#define GAME_DELAY 100
#define END_GAME_DELAY 3000

#define ON 1
#define OFF 0

#define TRUE 1
#define FALSE 0

#define PRESSED 1
#define NOT_PRESSED 0

#define P_UP PD0
#define P_LEFT PD1
#define P_DOWN PA6
#define P_RIGHT PD5

#define P_MARK PA7
#define P_HIT PD4

#include "includes/GLI.h"
#include "assets/font.h"
#include "assets/images.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>

typedef enum
{ 
	INITIAL = 0,
	CLEARED,
	QUESTION_MARK,
	FLAG
} state_t;

typedef enum
{
	NEW = 0,
	WIN,
	LOSS
} game_state_t;

typedef struct
{
	int has_mine;
	state_t state;
	int surrounding_mine_count;
	GLI_BITMAP *bm;
} cell_t;

cell_t cells[SCREEN_HEIGHT_SQUARENO][SCREEN_WIDTH_SQUARENO];

unsigned int seed, cell_count;
game_state_t game_state;

volatile int current_l = SCREEN_HEIGHT_SQUARENO / 2, prev_l;
volatile int current_c = SCREEN_WIDTH_SQUARENO / 2, prev_c;

volatile unsigned char mark_key_state;
volatile unsigned char hit_key_state; 

volatile unsigned char left_key_state;
volatile unsigned char up_key_state;
volatile unsigned char right_key_state;
volatile unsigned char down_key_state;

unsigned int left_key_pressed() 
{ 
	return !(PIND & (1 << P_LEFT));
}

unsigned int up_key_pressed() 
{ 
	return !(PIND & (1 << P_UP));
}

unsigned int right_key_pressed() 
{ 
	return !(PIND & (1 << P_RIGHT));
}

unsigned int down_key_pressed() 
{ 
	return !(PINA & (1 << P_DOWN));
}

unsigned int mark_key_pressed() 
{ 
	return !(PINA & (1 << P_MARK));
}

unsigned int hit_key_pressed() 
{ 
	return !(PIND & (1 << P_HIT));
}

/**
 * Handle buttons when pressed.
 */
void handle_pa_key_presses()
{	
	if(down_key_pressed())
		down_key_state = PRESSED;
	if(mark_key_pressed())
		mark_key_state = PRESSED;
}

void handle_pd_key_presses()
{
	if(left_key_pressed())
		left_key_state = PRESSED;
	if(up_key_pressed())
		up_key_state = PRESSED;
	if(right_key_pressed())
		right_key_state = PRESSED;
		
	if(hit_key_pressed())
		hit_key_state = PRESSED;
}

ISR(PCINT0_vect)
{
	handle_pa_key_presses();
}

ISR(PCINT3_vect)
{
	handle_pd_key_presses();
}

/**
 * Reset state of buttons.
 */
void reset_btns()
{
	up_key_state = NOT_PRESSED;
	down_key_state = NOT_PRESSED;
	left_key_state = NOT_PRESSED;
	right_key_state = NOT_PRESSED;
	
	mark_key_state = NOT_PRESSED;
	hit_key_state = NOT_PRESSED;
}

/**
 * Initializes the input buttons.
 */
void init_input()
{
	DDRD &= ~(1 << PD0);
	DDRD &= ~(1 << PD1);
	DDRA &= ~(1 << PA7);
	DDRA &= ~(1 << PD6);
	DDRD &= ~(1 << PD4);
	DDRD &= ~(1 << PD5);
	DDRD &= ~(1 << PD6);
	
	PORTD |= (1 << PD0);
	PORTD |= (1 << PD1);
	PORTA |= (1 << PA7);
	PORTA |= (1 << PA6);
	PORTD |= (1 << PD4);
	PORTD |= (1 << PD5);
	PORTD |= (1 << PD6);
	
	reset_btns();
}

/**
 * Start menu.
 */
void init_menu()
{
	char msg[BUFLEN];
	
	memset(msg, '0', BUFLEN);
	sprintf(msg, "- MINESWEEPER -");

	/* Prints the '- MINESWEEPER -' text message. */
	GLI_DrawString(
			(SCREEN_WIDTH - lucida_font.CharWidth * strlen( msg ) ) / 2 - 1,
			26,
			&lucida_font,
			msg
	);
	
	memset(msg, '0', BUFLEN);
	sprintf(msg, "PRESS BTN");

	/* Prints the 'PRESS BTN' text message. */
	GLI_DrawString(
			(SCREEN_WIDTH - lucida_font.CharWidth * strlen( msg ) ) / 2 - 1,
			38,
			&lucida_font,
			msg
	);
	GLI_Present();
}

/**
 * Generate the seed from which we will start placing mines.
 */
void init_generate_seed()
{
	while (1) {
	
		if ((PIND & (1 << PD6)) == 0)
			break;
	
		seed++;
	}
}

/**
 * For (l, c) cell get surrounding mine count.
 */
int get_surrounding_mine_count(int l, int c)
{
	int count = 0;
	
	if (l > 0 && c > 0 && cells[l - 1][c - 1].has_mine)
		count++;
	if (l > 0 && cells[l - 1][c].has_mine)
		count++;
	if (l > 0 && c < SCREEN_WIDTH_SQUARENO - 1 && cells[l - 1][c + 1].has_mine)
		count++;
		
	if (c > 0 && cells[l][c - 1].has_mine)
		count++;
	if (c < SCREEN_WIDTH_SQUARENO - 1 && cells[l][c + 1].has_mine)
		count++;
		
	if (l < SCREEN_HEIGHT_SQUARENO - 1 && c > 0 && cells[l + 1][c - 1].has_mine)
		count++;
	if (l < SCREEN_HEIGHT_SQUARENO - 1 && cells[l + 1][c].has_mine)
		count++;
	if (l < SCREEN_HEIGHT_SQUARENO - 1
		&& c < SCREEN_WIDTH_SQUARENO - 1 && cells[l + 1][c + 1].has_mine)
		count++;
		
	return count;
}

/**
 * Generate MINE_COUNT mines.
 */
void init_generate_mines()
{
	int l, c, mine_count = 0;
	cell_count = SCREEN_HEIGHT_SQUARENO * SCREEN_WIDTH_SQUARENO - MINE_COUNT;
	
	srand(seed);
	
	while (mine_count < MINE_COUNT) {
	
		l = rand() % SCREEN_HEIGHT_SQUARENO;
		c = rand() % SCREEN_WIDTH_SQUARENO;
		
		if (!cells[l][c].has_mine) {
		
			cells[l][c].has_mine = TRUE;
			cells[l][c].bm = &mine;
			mine_count++;
		}
	}
	
	/* update cells surrounding mines count */
	for (l = 0; l < SCREEN_HEIGHT_SQUARENO; l++)
		for (c = 0; c < SCREEN_WIDTH_SQUARENO; c++)
			if (!cells[l][c].has_mine)
				cells[l][c].surrounding_mine_count = get_surrounding_mine_count(l, c);
}

/**
 * Draw the borders and the filled cells and player start location.
 */
void init_draw_cells()
{
	unsigned char l, c;
	
	/* draw borders */
	for (l = 0; l < SCREEN_HEIGHT; l++)
		for (c = 0; c < SCREEN_WIDTH - 1; c++)
			if (l % 9 == 0 || c % 9 == 0)
				GLI_SetPixel(c, l, ON);

	/* draw filled cells */
	for (l = 0; l < SCREEN_HEIGHT_SQUARENO; l++)
		for (c = 0; c < SCREEN_WIDTH_SQUARENO; c++)
			if (l != current_l || c != current_c) {
				cells[l][c].bm = &filled_cell;
				GLI_DrawBitmap(
					c * (SQUARE_SIDE + 1) + 1,
					l * (SQUARE_SIDE + 1) + 1,
					cells[l][c].bm
				);
			}
	
	/* player start cell */
	cells[current_l][current_c].bm = &player_symbol;
	GLI_DrawBitmap(
		current_c * (SQUARE_SIDE + 1) + 1, 
		current_l * (SQUARE_SIDE + 1) + 1, 
		cells[current_l][current_c].bm
	);
	
	prev_l = current_l;
	prev_c = current_c;
	
	GLI_Present();
}

/**
 * Prepare game start.
 */
void init()
{
	GLI_Initialize();
	GLI_Clear(GLI_COLOR_BLACK);
	GLI_Present();
	
	init_input();
	
	init_menu();
	
	init_generate_seed();
	
	init_generate_mines();
	
	init_draw_cells();
}

/**
 * Set number in cleared cell.
 */
void set_cleared_cell_bitmap(int l, int c)
{

	switch (cells[l][c].surrounding_mine_count)
	{
		case 0:
			cells[l][c].bm = &empty_cell;
			break;
		case 1:
			cells[l][c].bm = &one_digit;
			break;
		case 2:
			cells[l][c].bm = &two_digit;
			break;
		case 3:
			cells[l][c].bm = &three_digit;
			break;
		case 4:
			cells[l][c].bm = &four_digit;
			break;
		case 5:
			cells[l][c].bm = &five_digit;
			break;
		case 6:
			cells[l][c].bm = &six_digit;
			break;
		case 7:
			cells[l][c].bm = &seven_digit;
			break;
		case 8:
			cells[l][c].bm = &eight_digit;
			break;
	}
}

/**
 * Handle user actions.
 */
void handle_input()
{
	handle_pa_key_presses();
	handle_pd_key_presses();
	
	prev_c = current_c;
	prev_l = current_l;
	
	if (left_key_state) {
	
		prev_c = current_c;
		if (current_c > 0)
			current_c--;
	}
	else if (up_key_state) {
		
		prev_l = current_l;
		if (current_l > 0)
			current_l--;
	}
	else if (right_key_state) {
	
		prev_c = current_c;
		if (current_c < SCREEN_WIDTH_SQUARENO - 1)
			current_c++;
	}
	else if (down_key_state) {
	
		prev_l = current_l;
		if (current_l < SCREEN_HEIGHT_SQUARENO - 1)
			current_l++;
	}
	else if (mark_key_state) {
	
		/* mark cell with flag or question mark */
	
		if (cells[current_l][current_c].state == CLEARED)
			return;
	
		switch (cells[current_l][current_c].state)
		{
			case INITIAL:
				cells[current_l][current_c].bm = &flag;
				cells[current_l][current_c].state = FLAG;
				break;
			case FLAG:
				cells[current_l][current_c].bm = &question_mark;
				cells[current_l][current_c].state = QUESTION_MARK;
				break;
			case QUESTION_MARK:
				cells[current_l][current_c].bm = &player_symbol;
				cells[current_l][current_c].state = INITIAL;; 
				break;
			default:
				break;
		}
	}
	else if (hit_key_state) {
	
		/* attempt to clear the cell */
	
		if (cells[current_l][current_c].state != INITIAL)
			return;
		
		if (cells[current_l][current_c].has_mine) {
		
			game_state = LOSS;
		}
		else {
		
			cells[current_l][current_c].state = CLEARED;
			set_cleared_cell_bitmap(current_l, current_c);
			cell_count--;
			
			if (cell_count == 0)
				game_state = WIN;
		}
	}
	
	if (prev_l != current_l || prev_c != current_c) {
	
		/* update previous cell with appropriate symbol */
	
		switch (cells[prev_l][prev_c].state)
		{
			case INITIAL:
				cells[prev_l][prev_c].bm = &filled_cell;
				break;
			case CLEARED:
				set_cleared_cell_bitmap(prev_l, prev_c);
				break;
			case QUESTION_MARK:
				cells[prev_l][prev_c].bm = &question_mark;
				break;
			case FLAG:
				cells[prev_l][prev_c].bm = &flag;
				break;
		}
		
		cells[current_l][current_c].bm = &player_symbol;
	}
}

/**
 * Update cell state and draw.
 */
void draw_screen()
{
	int l, c;
	char msg[BUFLEN];

	switch (game_state)
	{
	case LOSS:
	
		/* draw mines */
		for (l = 0; l < SCREEN_HEIGHT_SQUARENO; l++)
			for (c = 0; c < SCREEN_WIDTH_SQUARENO; c++)
				if (cells[l][c].has_mine)
					GLI_DrawBitmap(
						c * (SQUARE_SIDE + 1) + 1,
						l * (SQUARE_SIDE + 1) + 1,
						&mine
					);
		GLI_Present();	

		memset(msg, '0', BUFLEN);
		sprintf(msg, "YOU LOST! :(");

		/* Prints the 'YOU LOST! :(' text message. */
		GLI_Clear(GLI_COLOR_BLACK);
		GLI_DrawString(
				(SCREEN_WIDTH - lucida_font.CharWidth * strlen( msg ) ) / 2 - 1,
				26,
				&lucida_font,
				msg
		);
			
		_delay_ms(END_GAME_DELAY);
		break;
	case WIN:

		memset(msg, '0', BUFLEN);
		sprintf(msg, "YOU WON! :)");

		/* Prints the 'YOU WON! :)' text message. */
		GLI_Clear(GLI_COLOR_BLACK);
		GLI_DrawString(
				(SCREEN_WIDTH - lucida_font.CharWidth * strlen( msg ) ) / 2 - 1,
				26,
				&lucida_font,
				msg
		);
			
		_delay_ms(END_GAME_DELAY);
		break;
	default:
		/* update current & previous cells */
	
		if (prev_l != current_l || prev_c != current_c) {
	
			GLI_DrawBitmap(
				prev_c * (SQUARE_SIDE + 1) + 1, 
				prev_l * (SQUARE_SIDE + 1) + 1, 
				cells[prev_l][prev_c].bm
			);
		}
			
		GLI_DrawBitmap(
			current_c * (SQUARE_SIDE + 1) + 1, 
			current_l * (SQUARE_SIDE + 1) + 1, 
			cells[current_l][current_c].bm
		);
		break;
	}
	
	GLI_Present();
}

/**
 * Main function
 */
int main(void)
{

	/* init game */
	init();

	while(1)
	{
		/* treat user input */
		handle_input();
		
		/* reset buttons state */
		reset_btns();
		
		/* draw updates */
		draw_screen();
		
		/* check game state */
		if (game_state == LOSS || game_state ==  WIN)
			break;
		
		_delay_ms(GAME_DELAY);
	}

	return 0;
}
